@@ -18,13 +18,13 @@ from typing import Dict, Iterator, Optional, Set
 VarName = str
 
 from theano.compile import SharedVariable
-from theano.gof.graph import stack_search
+from theano.graph.basic import walk
 from theano.tensor import Tensor
 
 import pymc3 as pm
 
-from .model import ObservedRV
-from .util import get_default_varnames, get_var_name
+from pymc3.model import ObservedRV
+from pymc3.util import get_default_varnames, get_var_name
 
 
 class ModelGraph:
@@ -69,7 +69,7 @@ class ModelGraph:
             else:
                 return None
 
-        stack_search(start=deque([func]), expand=_expand, mode="bfs")
+        list(walk(deque([func]), _expand, bfs=True))
         return retval
 
     def _filter_parents(self, var, parents) -> Set[VarName]:
