@@ -97,7 +97,7 @@ from pymc3.distributions import (
 )
 from pymc3.math import kronecker, logsumexp
 from pymc3.model import Deterministic, Model, Point
-from pymc3.tests.helpers import SeededTest, select_by_precision
+from pymc3.tests.helpers import select_by_precision
 from pymc3.theanof import floatX
 from pymc3.vartypes import continuous_types
 
@@ -545,39 +545,75 @@ def RandomPdMatrix(n):
     return np.dot(A, A.T) + n * np.identity(n)
 
 
-class TestMatchesScipy(SeededTest):
-    def pymc3_matches_scipy(
+class TestMatchesScipy:
+    def check_logp(
         self,
         pymc3_dist,
         domain,
         paramdomains,
-        scipy_dist,
+        scipy_logp,
         decimal=None,
+        n_samples=100,
         extra_args=None,
         scipy_args=None,
     ):
+        """
+        Generic test for PyMC3 logp methods
+
+        Test PyMC3 logp and equivalent scipy logpmf/logpdf methods give similar
+        results for valid values and parameters inside the supported edges.
+        Edges are excluded by default, but can be artificially included by
+        creating a domain with repeated values (e.g., `Domain([0, 0, .5, 1, 1]`)
+
+        Parameters
+        ----------
+        pymc3_dist: PyMC3 distribution
+        domain : Domain
+            Supported domain of distribution values
+        paramdomains : Dictionary of Parameter : Domain pairs
+            Supported domains of distribution parameters
+        scipy_logp : Scipy logpmf/logpdf method
+            Scipy logp method of equivalent pymc3_dist distribution
+        decimal : Int
+            Level of precision with which pymc3_dist and scipy logp are compared.
+            Defaults to 6 for float64 and 3 for float32
+        n_samples : Int
+            Upper limit on the number of valid domain and value combinations that
+            are compared between pymc3 and scipy methods. If n_samples is below the
+            total number of combinations, a random subset is evaluated. Setting
+            n_samples = -1, will return all possible combinations. Defaults to 100
+        extra_args : Dictionary with extra arguments needed to build pymc3 model
+            Dictionary is passed to helper function `build_model` from which
+            the pymc3 distribution logp is calculated
+        scipy_args : Dictionary with extra arguments needed to call scipy logp method
+            Usually the same as extra_args
+        """
+        if decimal is None:
+            decimal = select_by_precision(float64=6, float32=3)
+
         if extra_args is None:
             extra_args = {}
+
         if scipy_args is None:
             scipy_args = {}
-        model = build_model(pymc3_dist, domain, paramdomains, extra_args)
-        value = model.named_vars["value"]
 
-        def logp(args):
+        def logp_reference(args):
             args.update(scipy_args)
-            return scipy_dist(**args)
+            return scipy_logp(**args)
 
-        self.check_logp(model, value, domain, paramdomains, logp, decimal=decimal)
+        model = build_model(pymc3_dist, domain, paramdomains, extra_args)
+        logp = model.fastlogp
 
-    def check_logp(self, model, value, domain, paramdomains, logp_reference, decimal=None):
         domains = paramdomains.copy()
         domains["value"] = domain
-        logp = model.fastlogp
-        for pt in product(domains, n_samples=100):
+        for pt in product(domains, n_samples=n_samples):
             pt = Point(pt, model=model)
-            if decimal is None:
-                decimal = select_by_precision(float64=6, float32=3)
-            assert_almost_equal(logp(pt), logp_reference(pt), decimal=decimal, err_msg=str(pt))
+            assert_almost_equal(
+                logp(pt),
+                logp_reference(pt),
+                decimal=decimal,
+                err_msg=str(pt),
+            )
 
     def check_logcdf(
         self,
@@ -621,8 +657,8 @@ class TestMatchesScipy(SeededTest):
         n_samples : Int
             Upper limit on the number of valid domain and value combinations that
             are compared between pymc3 and scipy methods. If n_samples is below the
-            total number of combinations, a random subset is evaluated.
-            Defaults to 100
+            total number of combinations, a random subset is evaluated. Setting
+            n_samples = -1, will return all possible combinations. Defaults to 100
         skip_paramdomain_inside_edge_test : Bool
             Whether to run test 1., which checks that pymc3 and scipy distributions
             match for valid values and parameters inside the respective domain edges
@@ -646,11 +682,12 @@ class TestMatchesScipy(SeededTest):
                 scipy_cdf = scipy_logcdf(**params)
                 value = params.pop("value")
                 dist = pymc3_dist.dist(**params)
+                params["value"] = value  # for displaying in err_msg
                 assert_almost_equal(
                     dist.logcdf(value).tag.test_value,
                     scipy_cdf,
                     decimal=decimal,
-                    err_msg=str(pt),
+                    err_msg=str(params),
                 )
 
         valid_value = domain.vals[0]
@@ -752,7 +789,7 @@ class TestMatchesScipy(SeededTest):
             check(m, m.named_vars["value"], valuedomain, vardomains)
 
     def test_uniform(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Uniform,
             Runif,
             {"lower": -Rplusunif, "upper": Rplusunif},
@@ -765,9 +802,13 @@ class TestMatchesScipy(SeededTest):
             lambda value, lower, upper: sp.uniform.logcdf(value, lower, upper - lower),
             skip_paramdomain_outside_edge_test=True,
         )
+        # Custom logp / logcdf check for invalid parameters
+        invalid_dist = Uniform.dist(lower=1, upper=0)
+        assert invalid_dist.logp(0.5).tag.test_value == -np.inf
+        assert invalid_dist.logcdf(2).tag.test_value == -np.inf
 
     def test_triangular(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Triangular,
             Runif,
             {"lower": -Rplusunif, "c": Runif, "upper": Rplusunif},
@@ -780,10 +821,18 @@ class TestMatchesScipy(SeededTest):
             lambda value, c, lower, upper: sp.triang.logcdf(value, c - lower, lower, upper - lower),
             skip_paramdomain_outside_edge_test=True,
         )
+        # Custom logp check for invalid value
+        valid_dist = Triangular.dist(lower=0, upper=1, c=2.0)
+        assert np.all(valid_dist.logp(np.array([1.9, 2.0, 2.1])).tag.test_value == -np.inf)
+
+        # Custom logp / logcdf check for invalid parameters
+        invalid_dist = Triangular.dist(lower=1, upper=0, c=2.0)
+        assert invalid_dist.logp(0.5).tag.test_value == -np.inf
+        assert invalid_dist.logcdf(2).tag.test_value == -np.inf
 
     def test_bound_normal(self):
         PositiveNormal = Bound(Normal, lower=0.0)
-        self.pymc3_matches_scipy(
+        self.check_logp(
             PositiveNormal,
             Rplus,
             {"mu": Rplus, "sigma": Rplus},
@@ -795,7 +844,7 @@ class TestMatchesScipy(SeededTest):
         assert np.isinf(x.logp({"x": -1}))
 
     def test_discrete_unif(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             DiscreteUniform,
             Rdunif,
             {"lower": -Rplusdunif, "upper": Rplusdunif},
@@ -813,9 +862,13 @@ class TestMatchesScipy(SeededTest):
             Rdunif,
             {"lower": -Rplusdunif, "upper": Rplusdunif},
         )
+        # Custom logp / logcdf check for invalid parameters
+        invalid_dist = DiscreteUniform.dist(lower=1, upper=0)
+        assert invalid_dist.logp(0.5).tag.test_value == -np.inf
+        assert invalid_dist.logcdf(2).tag.test_value == -np.inf
 
     def test_flat(self):
-        self.pymc3_matches_scipy(Flat, Runif, {}, lambda value: 0)
+        self.check_logp(Flat, Runif, {}, lambda value: 0)
         with Model():
             x = Flat("a")
             assert_allclose(x.tag.test_value, 0)
@@ -825,7 +878,7 @@ class TestMatchesScipy(SeededTest):
         assert -np.inf == Flat.dist().logcdf(-np.inf).tag.test_value
 
     def test_half_flat(self):
-        self.pymc3_matches_scipy(HalfFlat, Rplus, {}, lambda value: 0)
+        self.check_logp(HalfFlat, Rplus, {}, lambda value: 0)
         with Model():
             x = HalfFlat("a", shape=2)
             assert_allclose(x.tag.test_value, 1)
@@ -836,7 +889,7 @@ class TestMatchesScipy(SeededTest):
         assert -np.inf == HalfFlat.dist().logcdf(-np.inf).tag.test_value
 
     def test_normal(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Normal,
             R,
             {"mu": R, "sigma": Rplus},
@@ -856,7 +909,7 @@ class TestMatchesScipy(SeededTest):
                 value, (lower - mu) / sigma, (upper - mu) / sigma, loc=mu, scale=sigma
             )
 
-        self.pymc3_matches_scipy(
+        self.check_logp(
             TruncatedNormal,
             R,
             {"mu": R, "sigma": Rplusbig, "lower": -Rplusbig, "upper": Rplusbig},
@@ -865,7 +918,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_half_normal(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             HalfNormal,
             Rplus,
             {"sigma": Rplus},
@@ -880,7 +933,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_chi_squared(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             ChiSquared,
             Rplus,
             {"nu": Rplusdunif},
@@ -892,7 +945,7 @@ class TestMatchesScipy(SeededTest):
         reason="Poor CDF in SciPy. See scipy/scipy#869 for details.",
     )
     def test_wald_scipy(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Wald,
             Rplus,
             {"mu": Rplus, "alpha": Rplus},
@@ -936,18 +989,19 @@ class TestMatchesScipy(SeededTest):
         assert_almost_equal(model.fastlogp(pt), logp, decimal=decimals, err_msg=str(pt))
 
     def test_beta(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Beta,
             Unit,
             {"alpha": Rplus, "beta": Rplus},
             lambda value, alpha, beta: sp.beta.logpdf(value, alpha, beta),
         )
-        self.pymc3_matches_scipy(Beta, Unit, {"mu": Unit, "sigma": Rplus}, beta_mu_sigma)
+        self.check_logp(Beta, Unit, {"mu": Unit, "sigma": Rplus}, beta_mu_sigma)
         self.check_logcdf(
             Beta,
             Unit,
             {"alpha": Rplus, "beta": Rplus},
             lambda value, alpha, beta: sp.beta.logcdf(value, alpha, beta),
+            n_samples=10,
         )
 
     def test_kumaraswamy(self):
@@ -957,10 +1011,10 @@ class TestMatchesScipy(SeededTest):
                 np.log(a) + np.log(b) + (a - 1) * np.log(value) + (b - 1) * np.log(1 - value ** a)
             )
 
-        self.pymc3_matches_scipy(Kumaraswamy, Unit, {"a": Rplus, "b": Rplus}, scipy_log_pdf)
+        self.check_logp(Kumaraswamy, Unit, {"a": Rplus, "b": Rplus}, scipy_log_pdf)
 
     def test_exponential(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Exponential,
             Rplus,
             {"lam": Rplus},
@@ -974,7 +1028,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_geometric(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Geometric,
             Nat,
             {"p": Unit},
@@ -1010,7 +1064,7 @@ class TestMatchesScipy(SeededTest):
 
             return original_res if not np.isnan(original_res) else -np.inf
 
-        self.pymc3_matches_scipy(
+        self.check_logp(
             HyperGeometric,
             Nat,
             {"N": NatSmall, "k": NatSmall, "n": NatSmall},
@@ -1035,13 +1089,13 @@ class TestMatchesScipy(SeededTest):
         def scipy_mu_alpha_logcdf(value, mu, alpha):
             return sp.nbinom.logcdf(value, alpha, 1 - mu / (mu + alpha))
 
-        self.pymc3_matches_scipy(
+        self.check_logp(
             NegativeBinomial,
             Nat,
             {"mu": Rplus, "alpha": Rplus},
             scipy_mu_alpha_logpmf,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             NegativeBinomial,
             Nat,
             {"p": Unit, "n": Rplus},
@@ -1052,17 +1106,20 @@ class TestMatchesScipy(SeededTest):
             Nat,
             {"mu": Rplus, "alpha": Rplus},
             scipy_mu_alpha_logcdf,
+            n_samples=5,
         )
         self.check_logcdf(
             NegativeBinomial,
             Nat,
             {"p": Unit, "n": Rplus},
             lambda value, p, n: sp.nbinom.logcdf(value, n, p),
+            n_samples=5,
         )
         self.check_selfconsistency_discrete_logcdf(
             NegativeBinomial,
             Nat,
             {"mu": Rplus, "alpha": Rplus},
+            n_samples=10,
         )
 
     @pytest.mark.parametrize(
@@ -1086,7 +1143,7 @@ class TestMatchesScipy(SeededTest):
                 NegativeBinomial("x", mu=mu, p=p, alpha=alpha, n=n)
 
     def test_laplace(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Laplace,
             R,
             {"mu": R, "b": Rplus},
@@ -1100,7 +1157,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_laplace_asymmetric(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             AsymmetricLaplace,
             R,
             {"b": Rplus, "kappa": Rplus, "mu": R},
@@ -1108,7 +1165,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_lognormal(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Lognormal,
             Rplus,
             {"mu": R, "tau": Rplusbig},
@@ -1122,7 +1179,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_t(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             StudentT,
             R,
             {"nu": Rplus, "mu": R, "lam": Rplus},
@@ -1137,7 +1194,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_cauchy(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Cauchy,
             R,
             {"alpha": R, "beta": Rplusbig},
@@ -1151,7 +1208,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_half_cauchy(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             HalfCauchy,
             Rplus,
             {"beta": Rplusbig},
@@ -1164,8 +1221,8 @@ class TestMatchesScipy(SeededTest):
             lambda value, beta: sp.halfcauchy.logcdf(value, scale=beta),
         )
 
-    def test_gamma(self):
-        self.pymc3_matches_scipy(
+    def test_gamma_logp(self):
+        self.check_logp(
             Gamma,
             Rplus,
             {"alpha": Rplusbig, "beta": Rplusbig},
@@ -1175,11 +1232,21 @@ class TestMatchesScipy(SeededTest):
         def test_fun(value, mu, sigma):
             return sp.gamma.logpdf(value, mu ** 2 / sigma ** 2, scale=1.0 / (mu / sigma ** 2))
 
-        self.pymc3_matches_scipy(Gamma, Rplus, {"mu": Rplusbig, "sigma": Rplusbig}, test_fun)
+        self.check_logp(
+            Gamma,
+            Rplus,
+            {"mu": Rplusbig, "sigma": Rplusbig},
+            test_fun,
+        )
 
+    @pytest.mark.xfail(
+        condition=(theano.config.floatX == "float32"),
+        reason="Fails on float32 due to numerical issues",
+    )
+    def test_gamma_logcdf(self):
         # pymc-devs/Theano-PyMC#224: skip_paramdomain_outside_edge_test has to be set
         # True to avoid triggering a C-level assertion in the Theano GammaQ function
-        # in gamma.c file. Can be set back to False (defalut) once that issue is solved
+        # in gamma.c file. Can be set back to False (default) once that issue is solved
         self.check_logcdf(
             Gamma,
             Rplus,
@@ -1193,7 +1260,7 @@ class TestMatchesScipy(SeededTest):
         reason="Fails on float32 due to numerical issues",
     )
     def test_inverse_gamma(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             InverseGamma,
             Rplus,
             {"alpha": Rplus, "beta": Rplus},
@@ -1201,7 +1268,7 @@ class TestMatchesScipy(SeededTest):
         )
         # pymc-devs/Theano-PyMC#224: skip_paramdomain_outside_edge_test has to be set
         # True to avoid triggering a C-level assertion in the Theano GammaQ function
-        # in gamma.c file. Can be set back to False (defalut) once that issue is solved
+        # in gamma.c file. Can be set back to False (default) once that issue is solved
         self.check_logcdf(
             InverseGamma,
             Rplus,
@@ -1219,10 +1286,16 @@ class TestMatchesScipy(SeededTest):
             alpha, beta = InverseGamma._get_alpha_beta(None, None, mu, sigma)
             return sp.invgamma.logpdf(value, alpha, scale=beta)
 
-        self.pymc3_matches_scipy(InverseGamma, Rplus, {"mu": Rplus, "sigma": Rplus}, test_fun)
+        self.check_logp(
+            InverseGamma,
+            Rplus,
+            {"mu": Rplus, "sigma": Rplus},
+            test_fun,
+            decimal=select_by_precision(float64=5, float32=3),
+        )
 
     def test_pareto(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Pareto,
             Rplus,
             {"alpha": Rplusbig, "m": Rplusbig},
@@ -1240,7 +1313,7 @@ class TestMatchesScipy(SeededTest):
         reason="Fails on float32 due to inf issues",
     )
     def test_weibull(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Weibull,
             Rplus,
             {"alpha": Rplusbig, "beta": Rplusbig},
@@ -1255,7 +1328,7 @@ class TestMatchesScipy(SeededTest):
 
     def test_half_studentt(self):
         # this is only testing for nu=1 (halfcauchy)
-        self.pymc3_matches_scipy(
+        self.check_logp(
             HalfStudentT,
             Rplus,
             {"sigma": Rplus},
@@ -1263,15 +1336,16 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_skew_normal(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             SkewNormal,
             R,
             {"mu": R, "sigma": Rplusbig, "alpha": R},
             lambda value, alpha, mu, sigma: sp.skewnorm.logpdf(value, alpha, mu, sigma),
+            decimal=select_by_precision(float64=5, float32=3),
         )
 
     def test_binomial(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Binomial,
             Nat,
             {"n": NatSmall, "p": Unit},
@@ -1282,11 +1356,13 @@ class TestMatchesScipy(SeededTest):
             Nat,
             {"n": NatSmall, "p": Unit},
             lambda value, n, p: sp.binom.logcdf(value, n, p),
+            n_samples=10,
         )
         self.check_selfconsistency_discrete_logcdf(
             Binomial,
             Nat,
             {"n": NatSmall, "p": Unit},
+            n_samples=10,
         )
 
     # Too lazy to propagate decimal parameter through the whole chain of deps
@@ -1300,7 +1376,7 @@ class TestMatchesScipy(SeededTest):
             Nat,
             {"alpha": Rplus, "beta": Rplus, "n": NatSmall},
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             BetaBinomial,
             Nat,
             {"alpha": Rplus, "beta": Rplus, "n": NatSmall},
@@ -1319,13 +1395,13 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_bernoulli(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Bernoulli,
             Bool,
             {"logit_p": R},
             lambda value, logit_p: sp.bernoulli.logpmf(value, scipy.special.expit(logit_p)),
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Bernoulli,
             Bool,
             {"p": Unit},
@@ -1350,7 +1426,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_discrete_weibull(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             DiscreteWeibull,
             Nat,
             {"q": Unit, "beta": Rplusdunif},
@@ -1363,7 +1439,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_poisson(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Poisson,
             Nat,
             {"mu": Rplus},
@@ -1383,7 +1459,7 @@ class TestMatchesScipy(SeededTest):
 
     def test_bound_poisson(self):
         NonZeroPoisson = Bound(Poisson, lower=1.0)
-        self.pymc3_matches_scipy(
+        self.check_logp(
             NonZeroPoisson,
             PosNat,
             {"mu": Rplus},
@@ -1395,7 +1471,7 @@ class TestMatchesScipy(SeededTest):
         assert np.isinf(x.logp({"x": 0}))
 
     def test_constantdist(self):
-        self.pymc3_matches_scipy(Constant, I, {"c": I}, lambda value, c: np.log(c == value))
+        self.check_logp(Constant, I, {"c": I}, lambda value, c: np.log(c == value))
 
     # Too lazy to propagate decimal parameter through the whole chain of deps
     @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
@@ -1423,6 +1499,7 @@ class TestMatchesScipy(SeededTest):
             ZeroInflatedNegativeBinomial,
             Nat,
             {"mu": Rplusbig, "alpha": Rplusbig, "psi": Unit},
+            n_samples=10,
         )
 
     # Too lazy to propagate decimal parameter through the whole chain of deps
@@ -1437,42 +1514,43 @@ class TestMatchesScipy(SeededTest):
             ZeroInflatedBinomial,
             Nat,
             {"n": NatSmall, "p": Unit, "psi": Unit},
+            n_samples=10,
         )
 
     @pytest.mark.parametrize("n", [1, 2, 3])
     def test_mvnormal(self, n):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvNormal,
             RealMatrix(5, n),
             {"mu": Vector(R, n), "tau": PdMatrix(n)},
             normal_logpdf_tau,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvNormal,
             Vector(R, n),
             {"mu": Vector(R, n), "tau": PdMatrix(n)},
             normal_logpdf_tau,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvNormal,
             RealMatrix(5, n),
             {"mu": Vector(R, n), "cov": PdMatrix(n)},
             normal_logpdf_cov,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvNormal,
             Vector(R, n),
             {"mu": Vector(R, n), "cov": PdMatrix(n)},
             normal_logpdf_cov,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvNormal,
             RealMatrix(5, n),
             {"mu": Vector(R, n), "chol": PdMatrixChol(n)},
             normal_logpdf_chol,
             decimal=select_by_precision(float64=6, float32=-1),
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvNormal,
             Vector(R, n),
             {"mu": Vector(R, n), "chol": PdMatrixChol(n)},
@@ -1483,7 +1561,7 @@ class TestMatchesScipy(SeededTest):
         def MvNormalUpper(*args, **kwargs):
             return MvNormal(lower=False, *args, **kwargs)
 
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvNormalUpper,
             Vector(R, n),
             {"mu": Vector(R, n), "chol": PdMatrixCholUpper(n)},
@@ -1527,7 +1605,7 @@ class TestMatchesScipy(SeededTest):
     def test_matrixnormal(self, n):
         mat_scale = 1e3  # To reduce logp magnitude
         mean_scale = 0.1
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MatrixNormal,
             RealMatrix(n, n),
             {
@@ -1537,7 +1615,7 @@ class TestMatchesScipy(SeededTest):
             },
             matrix_normal_logpdf_cov,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MatrixNormal,
             RealMatrix(2, n),
             {
@@ -1547,7 +1625,7 @@ class TestMatchesScipy(SeededTest):
             },
             matrix_normal_logpdf_cov,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MatrixNormal,
             RealMatrix(3, n),
             {
@@ -1558,7 +1636,7 @@ class TestMatchesScipy(SeededTest):
             matrix_normal_logpdf_chol,
             decimal=select_by_precision(float64=6, float32=-1),
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MatrixNormal,
             RealMatrix(n, 3),
             {
@@ -1592,7 +1670,7 @@ class TestMatchesScipy(SeededTest):
             for args in [cov_args, chol_args, evd_args]:
                 args["sigma"] = sigma
 
-        self.pymc3_matches_scipy(
+        self.check_logp(
             KroneckerNormal,
             dom,
             std_args,
@@ -1600,7 +1678,7 @@ class TestMatchesScipy(SeededTest):
             extra_args=cov_args,
             scipy_args=cov_args,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             KroneckerNormal,
             dom,
             std_args,
@@ -1608,7 +1686,7 @@ class TestMatchesScipy(SeededTest):
             extra_args=chol_args,
             scipy_args=chol_args,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             KroneckerNormal,
             dom,
             std_args,
@@ -1619,7 +1697,7 @@ class TestMatchesScipy(SeededTest):
 
         dom = Domain([np.random.randn(2, N) * 0.1], edges=(None, None), shape=(2, N))
 
-        self.pymc3_matches_scipy(
+        self.check_logp(
             KroneckerNormal,
             dom,
             std_args,
@@ -1627,7 +1705,7 @@ class TestMatchesScipy(SeededTest):
             extra_args=cov_args,
             scipy_args=cov_args,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             KroneckerNormal,
             dom,
             std_args,
@@ -1635,7 +1713,7 @@ class TestMatchesScipy(SeededTest):
             extra_args=chol_args,
             scipy_args=chol_args,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             KroneckerNormal,
             dom,
             std_args,
@@ -1646,13 +1724,13 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [1, 2])
     def test_mvt(self, n):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvStudentT,
             Vector(R, n),
             {"nu": Rplus, "Sigma": PdMatrix(n), "mu": Vector(R, n)},
             mvt_logpdf,
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             MvStudentT,
             RealMatrix(2, n),
             {"nu": Rplus, "Sigma": PdMatrix(n), "mu": Vector(R, n)},
@@ -1661,7 +1739,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3, 4])
     def test_AR1(self, n):
-        self.pymc3_matches_scipy(AR1, Vector(R, n), {"k": Unit, "tau_e": Rplus}, AR1_logpdf)
+        self.check_logp(AR1, Vector(R, n), {"k": Unit, "tau_e": Rplus}, AR1_logpdf)
 
     @pytest.mark.parametrize("n", [2, 3])
     def test_wishart(self, n):
@@ -1684,9 +1762,20 @@ class TestMatchesScipy(SeededTest):
         decimals = select_by_precision(float64=6, float32=4)
         assert_almost_equal(model.fastlogp(pt), lp, decimal=decimals, err_msg=str(pt))
 
-    @pytest.mark.parametrize("n", [2, 3])
+    @pytest.mark.parametrize("n", [1, 2, 3])
     def test_dirichlet(self, n):
-        self.pymc3_matches_scipy(Dirichlet, Simplex(n), {"a": Vector(Rplus, n)}, dirichlet_logpdf)
+        self.check_logp(Dirichlet, Simplex(n), {"a": Vector(Rplus, n)}, dirichlet_logpdf)
+
+    @pytest.mark.parametrize("dist_shape", [1, (2, 1), (1, 2), (2, 4, 3)])
+    def test_dirichlet_with_batch_shapes(self, dist_shape):
+        a = np.ones(dist_shape)
+        with pm.Model() as model:
+            d = pm.Dirichlet("a", a=a)
+
+        pymc3_res = d.distribution.logp(d.tag.test_value).eval()
+        for idx in np.ndindex(a.shape[:-1]):
+            scipy_res = scipy.stats.dirichlet(a[idx]).logpdf(d.tag.test_value[idx])
+            assert_almost_equal(pymc3_res[idx], scipy_res)
 
     def test_dirichlet_shape(self):
         a = tt.as_tensor_variable(np.r_[1, 2])
@@ -1698,7 +1787,7 @@ class TestMatchesScipy(SeededTest):
             dir_rv = Dirichlet.dist(tt.vector())
 
     def test_dirichlet_2D(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Dirichlet,
             MultiSimplex(2, 2),
             {"a": Vector(Vector(Rplus, 2), 2)},
@@ -1707,7 +1796,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3])
     def test_multinomial(self, n):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Multinomial, Vector(Nat, n), {"p": Simplex(n), "n": Nat}, multinomial_logpdf
         )
 
@@ -1857,7 +1946,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3])
     def test_dirichlet_multinomial(self, n):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             DirichletMultinomial,
             Vector(Nat, n),
             {"a": Vector(Rplus, n), "n": Nat},
@@ -2025,7 +2114,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3, 4])
     def test_categorical(self, n):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Categorical,
             Domain(range(n), "int64"),
             {"p": Simplex(n)},
@@ -2034,7 +2123,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3, 4])
     def test_orderedlogistic(self, n):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             OrderedLogistic,
             Domain(range(n), "int64"),
             {"eta": R, "cutpoints": Vector(R, n - 1)},
@@ -2043,7 +2132,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.parametrize("n", [2, 3, 4])
     def test_orderedprobit(self, n):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             OrderedProbit,
             Domain(range(n), "int64"),
             {"eta": Runif, "cutpoints": UnitSortedVector(n - 1)},
@@ -2126,7 +2215,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_vonmises(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             VonMises,
             R,
             {"mu": Circ, "kappa": Rplus},
@@ -2137,7 +2226,7 @@ class TestMatchesScipy(SeededTest):
         def gumbel(value, mu, beta):
             return floatX(sp.gumbel_r.logpdf(value, loc=mu, scale=beta))
 
-        self.pymc3_matches_scipy(Gumbel, R, {"mu": R, "beta": Rplusbig}, gumbel)
+        self.check_logp(Gumbel, R, {"mu": R, "beta": Rplusbig}, gumbel)
 
         def gumbellcdf(value, mu, beta):
             return floatX(sp.gumbel_r.logcdf(value, loc=mu, scale=beta))
@@ -2145,7 +2234,7 @@ class TestMatchesScipy(SeededTest):
         self.check_logcdf(Gumbel, R, {"mu": R, "beta": Rplusbig}, gumbellcdf)
 
     def test_logistic(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Logistic,
             R,
             {"mu": R, "s": Rplus},
@@ -2161,7 +2250,7 @@ class TestMatchesScipy(SeededTest):
         )
 
     def test_logitnormal(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             LogitNormal,
             Unit,
             {"mu": R, "sigma": Rplus},
@@ -2176,13 +2265,13 @@ class TestMatchesScipy(SeededTest):
             Beta("beta", alpha=1.0, beta=1.0, shape=(10, 20))
 
     def test_rice(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Rice,
             Rplus,
             {"nu": Rplus, "sigma": Rplusbig},
             lambda value, nu, sigma: sp.rice.logpdf(value, b=nu / sigma, loc=0, scale=sigma),
         )
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Rice,
             Rplus,
             {"b": Rplus, "sigma": Rplusbig},
@@ -2191,7 +2280,7 @@ class TestMatchesScipy(SeededTest):
 
     @pytest.mark.xfail(condition=(theano.config.floatX == "float32"), reason="Fails on float32")
     def test_moyal(self):
-        self.pymc3_matches_scipy(
+        self.check_logp(
             Moyal,
             R,
             {"mu": R, "sigma": Rplusbig},
@@ -2225,7 +2314,7 @@ class TestMatchesScipy(SeededTest):
                         -np.inf * np.ones(value.shape),
                     )
 
-                self.pymc3_matches_scipy(TestedInterpolated, R, {}, ref_pdf)
+                self.check_logp(TestedInterpolated, R, {}, ref_pdf)
 
 
 def test_bound():
@@ -2507,7 +2596,7 @@ def test_serialize_density_dist():
     with pm.Model():
         pm.Normal("x")
         y = pm.DensityDist("y", func)
-        pm.sample(draws=5, tune=1, mp_ctx="spawn")
+        pm.sample(draws=5, tune=1, mp_ctx="spawn", return_inferencedata=False)
 
     import pickle
 
