@@ -107,15 +107,12 @@ def pymc3_random_discrete(
             e = ref_rand(size=size, **pt)
             o = np.atleast_1d(o).flatten()
             e = np.atleast_1d(e).flatten()
-            observed = dict(zip(*np.unique(o, return_counts=True)))
-            expected = dict(zip(*np.unique(e, return_counts=True)))
-            for e in expected.keys():
-                expected[e] = (observed.get(e, 0), expected[e])
-            k = np.array([v for v in expected.values()])
-            if np.all(k[:, 0] == k[:, 1]):
+            observed, _ = np.histogram(o, bins=min(7, len(set(o))))
+            expected, _ = np.histogram(e, bins=min(7, len(set(o))))
+            if np.all(observed == expected):
                 p = 1.0
             else:
-                _, p = st.chisquare(k[:, 0], k[:, 1])
+                _, p = st.chisquare(observed + 1, expected + 1)
             f -= 1
         assert p > alpha, str(pt)
 
@@ -1296,7 +1293,7 @@ class TestDensityDist:
                 shape=shape,
                 random=normal_dist.random,
             )
-            trace = pm.sample(100, cores=1)
+            trace = pm.sample(100, cores=1, return_inferencedata=False)
 
         samples = 500
         size = 100
@@ -1319,7 +1316,7 @@ class TestDensityDist:
                 random=normal_dist.random,
                 wrap_random_with_dist_shape=False,
             )
-            trace = pm.sample(100, cores=1)
+            trace = pm.sample(100, cores=1, return_inferencedata=False)
 
         samples = 500
         with pytest.raises(RuntimeError):
@@ -1342,7 +1339,7 @@ class TestDensityDist:
                 wrap_random_with_dist_shape=False,
                 check_shape_in_random=False,
             )
-            trace = pm.sample(100, cores=1)
+            trace = pm.sample(100, cores=1, return_inferencedata=False)
 
         samples = 500
         ppc = pm.sample_posterior_predictive(trace, samples=samples, model=model)
@@ -1365,7 +1362,7 @@ class TestDensityDist:
                 random=rvs,
                 wrap_random_with_dist_shape=False,
             )
-            trace = pm.sample(100, cores=1)
+            trace = pm.sample(100, cores=1, return_inferencedata=False)
 
         samples = 500
         size = 100
@@ -1385,7 +1382,7 @@ class TestDensityDist:
                 random=rvs,
                 wrap_random_with_dist_shape=False,
             )
-            trace = pm.sample(100, cores=1)
+            trace = pm.sample(100, cores=1, return_inferencedata=False)
 
         samples = 500
         size = 100
@@ -1398,7 +1395,7 @@ class TestDensityDist:
             mu = pm.Normal("mu", 0, 1)
             normal_dist = pm.Normal.dist(mu, 1)
             pm.DensityDist("density_dist", normal_dist.logp, observed=np.random.randn(100))
-            trace = pm.sample(100, cores=1)
+            trace = pm.sample(100, cores=1, return_inferencedata=False)
 
         samples = 500
         with pytest.raises(ValueError):
@@ -1823,6 +1820,21 @@ def test_matrix_normal_random_with_random_variables():
     assert prior["mu"].shape == (2, D, K)
 
 
+@pytest.mark.parametrize("n", [2, 5])
+def test_lkj_corr_with_prior_predictive(n):
+    """
+    This test checks for shape correctness when using pm.sample_prior_predictive
+    on LKJCorr distribution.
+    Originally reported - https://github.com/pymc-devs/pymc3/issues/4778
+    """
+    with pm.Model():
+        lkj = pm.LKJCorr("lkj", eta=1, n=n)
+        prior = pm.sample_prior_predictive(500)
+
+    upper_triangular_shape = n * (n - 1) // 2
+    assert prior["lkj"].shape == (500, upper_triangular_shape)
+
+
 class TestMvGaussianRandomWalk(SeededTest):
     @pytest.mark.parametrize(
         ["sample_shape", "dist_shape", "mu_shape", "param"],
